<?php

namespace Drupal\override_cache_control_header\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Set the configured cache control headers against the provided URL.
 */
class OverrideCacheControlHeader implements EventSubscriberInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Override cache control header constructor.
   *
   * Initialize the config factory object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory service.
   */
  public function __construct(ConfigFactoryInterface $configFactory) {
    $this->configFactory = $configFactory;
  }

  /**
   * Modifies the response to disable cache for a specific route.
   *
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   The response event that contains the request and response objects.
   */
  public function onViewResponse(ResponseEvent $event) {
    // Check if the request path matches the specific route.
    $urls_header = $this->configFactory->get('override_cache_control_header.settings')->get('urls_header');
    if (!empty($urls_header)) {
      $urls_header = trim($urls_header);
      $urls_headers = explode(PHP_EOL, $urls_header);
      if ((is_array($urls_headers)) && (!empty($urls_headers))) {
        $urls_headers = array_filter($urls_headers);
        $this->setHeaders($urls_headers, $event);
      }
    }
  }

  /**
   * Sets the headers for the given URLs.
   *
   * @param array $urls_headers
   *   An array of URLs for which the headers need to be set.
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   The response event that contains the request and response objects.
   */
  private function setHeaders(array $urls_headers, ResponseEvent $event) {
    $pathinfo = $event->getRequest()->getPathInfo();
    foreach ($urls_headers as $urls_header) {
      $exploded = explode('|', $urls_header);
      if (str_starts_with($pathinfo, $exploded[0])) {
        $response = $event->getResponse();
        // Set Cache-Control header to disable.
        // caching for this specific route.
        $response->headers->set('Cache-Control', $exploded[1]);
        return;
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events = [];
    $events[KernelEvents::RESPONSE][] = ['onViewResponse', 10000];
    return $events;
  }

}
