<?php

namespace Drupal\override_cache_control_header;

use Drupal\Core\Path\PathValidator;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Class to handle the helper functions.
 */
class Utility {
  use StringTranslationTrait;

  /**
   * Path validator object.
   *
   * @var \Drupal\Core\Path\PathValidator
   */
  protected $pathValidator;

  /**
   * Utility class contructor.
   *
   * @param \Drupal\Core\Path\PathValidator $pathValidator
   *   Path validator object.
   */
  public function __construct(PathValidator $pathValidator) {
    $this->pathValidator = $pathValidator;
  }

  /**
   * Validate the cache control headers and URLs.
   *
   * @param array $urls_headers
   *   List of URLs and cache control headers.
   */
  public function validateCacheControlStrings(array $urls_headers): array {
    // Regular expression pattern for validation.
    $pattern = '#^(/[a-zA-Z0-9_\-/.]+)\|([a-zA-Z0-9\- ,=]+)$#';
    $response = [];
    $urls_headers = array_filter($urls_headers);
    foreach ($urls_headers as $urls_header) {
      if (preg_match($pattern, $urls_header, $matches)) {
        // Extracted URL.
        $url = $matches[1];
        // Extracted cache-control header.
        $cacheControl = $matches[2];
        // Validate the URL is valid internal URL.
        if (!$this->pathValidator->isValid($url)) {
          $response[] = $this->t('This @url is not a valid internal URL.', ['@url' => $url]);
        }
        // List of valid Cache-Control directives.
        $validDirectives = [
          'must-revalidate',
          'no-cache',
          'no-store',
          'public',
          'private',
          'proxy-revalidate',
          'max-age',
          's-maxage',
        ];
        // Split cache-control string and validate each directive.
        $directives = array_map('trim', explode(',', $cacheControl));
        if ((is_array($directives)) && (!empty($directives))) {
          foreach ($directives as $directive) {
            $directiveParts = explode('=', $directive);
            if (!in_array($directiveParts[0], $validDirectives)) {
              $response[] = $this->t(
                'This @directive cache-control directive is invalid.',
                ['@directive' => $directiveParts[0]]
              );
            }
          }
        }
      }
      else {
        $response[] = $this->t('This @urls_header format is invalid.', ['@urls_header' => $urls_header]);
      }
    }
    return $response;
  }

}
