<?php

namespace Drupal\Tests\override_cache_control_header\Unit;

use Drupal\Core\Path\PathValidator;
use Drupal\override_cache_control_header\Utility;
use Drupal\Tests\UnitTestCase;


/**
 * Unit test for the Utility class.
 *
 * @group override_cache_control_header
 */
class UtilityTest extends UnitTestCase {

  /**
   * The Utility class instance.
   *
   * @var \Drupal\override_cache_control_header\Utility
   */
  protected $utility;

  /**
   * Pathvalidator
   * 
   * @var \Drupal\Core\Path\PathValidator
   */
  protected $pathValidator;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Mock the PathValidator service.
    $this->pathValidator = $this->getMockBuilder(PathValidator::class)
                            ->disableOriginalConstructor()
                            ->setMethods(['isValid'])
                            ->getMock();

    // Mock the StringTranslationTrait.
    $this->utility = $this->getMockBuilder(Utility::class)
      ->setConstructorArgs([$this->pathValidator])
      ->onlyMethods(['t'])
      ->getMock();

    // Mock the translation method.
    $this->utility->method('t')
      ->willReturnCallback(function ($string, array $args = []) {
        return strtr($string, $args);
      });
  }

  /**
   * Tests validateCacheControlStrings with valid input.
   */
  public function testValidateCacheControlStringsValid() {
    $urls_headers = [
      '/valid-path|public, max-age=3600',
      '/another-valid-path|private, no-cache',
    ];
    $this->pathValidator->expects($this->any())->method('isValid')->willReturn(true);
    $result = $this->utility->validateCacheControlStrings($urls_headers);
    $this->assertEmpty($result, 'No errors should be returned for valid input.');
  }

  /**
   * Tests validateCacheControlStrings with invalid URL.
   */
  public function testValidateCacheControlStringsInvalidUrl() {
    $urls_headers = [
      'invalid-path|public, max-age=3600',
    ];
    $this->pathValidator->expects($this->any())->method('isValid')->willReturn(false);
    $result = $this->utility->validateCacheControlStrings($urls_headers);
    $this->assertNotEmpty($result, 'Errors should be returned for invalid URL.');
    $this->assertStringContainsString('This invalid-path|public, max-age=3600 format is invalid.', $result[0]);
  }

  /**
   * Tests validateCacheControlStrings with invalid cache-control directive.
   */
  public function testValidateCacheControlStringsInvalidDirective() {
    $urls_headers = [
      '/valid-path|invalid-directive',
    ];
    $this->pathValidator->expects($this->any())->method('isValid')->willReturn(true);
    $result = $this->utility->validateCacheControlStrings($urls_headers);
    $this->assertNotEmpty($result, 'Errors should be returned for invalid cache-control directive.');
    $this->assertStringContainsString('This invalid-directive cache-control directive is invalid.', $result[0]);
  }

  /**
   * Tests validateCacheControlStrings with invalid format.
   */
  public function testValidateCacheControlStringsInvalidFormat() {
    $urls_headers = [
      '/valid-path public, max-age=3600',
    ];
    $this->pathValidator->method('isValid')->willReturn(false);
    $result = $this->utility->validateCacheControlStrings($urls_headers);
    $this->assertNotEmpty($result, 'Errors should be returned for invalid format.');
    $this->assertStringContainsString('This /valid-path public, max-age=3600 format is invalid.', $result[0]);
  }

  /**
   * Tests validateCacheControlStrings with invalid internal URL.
   */
  public function testValidateCacheControlStringsInvalidInternalUrl() {
    $urls_headers = [
      '/invalid-internal-path|public, max-age=3600',
    ];
    $this->pathValidator->method('isValid')->willReturn(false);
    $result = $this->utility->validateCacheControlStrings($urls_headers);
    $this->assertNotEmpty($result, 'Errors should be returned for invalid internal URL.');
    $this->assertStringContainsString('This /invalid-internal-path is not a valid internal URL.', $result[0]);
  }
}
